/* 
 * Binary search implementation
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.1.1.1 $  $Date: 2003/01/10 23:15:54 $
 * ----------------------------------------------------------------------
 */

#include <phlib/xl_config.h>
#include <phlib/xl_types.h>
#include <phlib/xl_string.h>
#include <phlib/xl_dbeng.h>

/* ---------------------------------------------------------------------- */
/* Binary serach for Records of type dbeng
 *
 * Entry: CString wrapped around DataBase data (Must Be FRESH (p = s) )
 *        CString wrapped around search key
 *        size of record
 *
 * Exit : Nearest match to record requested (offset into data passed in)
 *
 */
int
xl_Binary_srch( CString data, CString key, int rec_size )
{
  int     recs;
  int     mid;
  int     res;
  CString midrec;
  CString dataset;
  CString reckey;

  MirrorCString( midrec, data ); 

  if( data.len >= rec_size ) { 

    MirrorCString( dataset, data );
    
    recs = data.len / rec_size;     /* number of records */
    mid  = recs / 2;                /* middle of records */
    mid *= rec_size;                /* Actual position of middle record */
    
    string_mid( &midrec, mid, mid + rec_size ); /* Grab middle record */
    
    /* Extract key value from mid_record */
    WrapCStringLen( reckey, PTR_KEY_START(midrec), VAL_KEY_LEN(midrec) );
    
    if( (res = CScmpAlphabetic( &reckey, &key )) < 0 ) {   /* reckey < key */
      
      /* Take next record to the end */
      string_right( &dataset, dataset.len - mid - rec_size );
      return xl_Binary_srch( dataset, key, rec_size );
      
    } else if( res > 0 ) { /* reckey > key */
      
      /* Take from start to the previous record */
      string_left( &dataset, mid );
      return xl_Binary_srch( dataset, key, rec_size );
    }
  }

  /* Done */
  return (midrec.s - midrec.p);
}

/* ---------------------------------------------------------------------- */
/* end */
