/*
 * ------------------------------------------------------------------------
 * Module : Simple Resource File Reader
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.1.1.1 $  $Date: 2003/01/10 23:16:02 $
 * ----------------------------------------------------------------------- 
 */

#include <phlib/xl_config.h>
#include <phlib/xl_types.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <phlib/xl_string.h>
#include <phlib/xl_mmap.h>

static char comment = '#';
static char LF      = '\n';

#define RESMAX 8192
static char fname     [256]      = "";
static char resource  [RESMAX+1] = "";  /* Where resource data is copied */
static const char HOME[]         = "HOME";

static char *file_search( CString *, const char* );

/* ----------------------------------------------------------------------- */

char *
xl_readrc( const char *res, const char *rcfile )
{
  char    *ptr, *value = 0;
  char    *rfile = (char*)rcfile;
  int     l = strlen( rcfile );
  CString dataCS = CSInit;

  do {
    /* Are we looking for an implicit file */
    if( rfile[0] == '/' ) {
      if( xl_mapfile( rfile, &dataCS ) >= 0 ) {
        value = file_search( &dataCS, res );
        break;
      } else {
	/* Chop off filename from string */
	rfile = (char*)rcfile + l;
	while( rfile >= rcfile && *rfile != '/' ) rfile--;
	rfile++;
      }
    }

    /* Check current directory for rfile */
    strcpy( fname, "./"  );
    strcat( fname, rfile );
      
    if( xl_mapfile( fname, &dataCS ) >= 0 ) {
      value = file_search( &dataCS, res );
      break;
    }

    /* Else check home directory ? */
    if( (ptr = getenv( HOME )) != NULL ) {
      strcpy( fname, ptr );
      strcat( fname, "/" );
      strcat( fname, rfile );

      if( xl_mapfile( fname, &dataCS ) >= 0 ) {
	value = file_search( &dataCS, res );
	break;
      }
    }


  } while(0);

  if( dataCS.p != 0 ) xl_unmapfile( &dataCS );

  return value;
}


/* ----------------------------------------------------------------------- */

static char *
file_search( CString *dataCS, const char* res )
{
  CString lineCS = CSInit;
  CString resCS  = CSInit;
  CString srchCS = CSInit;
  char   *ret   = NULL;

  WrapCString( srchCS, res );

  while( chop_line( &lineCS, dataCS, LF ) == Success ) {
    
    /* Skip blank or lines beginning with a # */
    skip_lead_space( &lineCS );
    if( CSLen( lineCS ) == 0 || CSChar( lineCS, 0 ) == comment ) continue;

    /* Match up resource */
    chop_line( &resCS, &lineCS, '=' );
    skip_tail_space( &resCS );

    if( CScmp( &resCS, &srchCS ) == 0 ) { /* Found */
      skip_lead_space( &lineCS );
      skip_tail_space( &lineCS );
      CStoCharCpy( resource, &lineCS, RESMAX ); 
      ret = resource;
    }
  }
  return ret;
}


/* ----------------------------------------------------------------------- */
/* end */
