/*
 * Unique Filename Generation
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.1.1.1 $  $Date: 2003/01/10 23:16:04 $
 * ---------------------------------------------------------------------------
 */

#include <phlib/xl_config.h>

#if HAVE_UNISTD_H
# include <sys/types.h>
# include <unistd.h>
#endif

#include <stdio.h>
#include <fcntl.h>      /* File control - file locks */
#include <sys/stat.h>   /* File creation modes */ 

#ifndef HAVE_FLOCK_T
typedef struct flock flock_t;
#endif

#if STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_MEMCPY
#  define memcpy(d, s, n) bcopy ((s), (d), (n))
#  define memmove(d, s, n) bcopy ((s), (d), (n))
# endif
#endif

#include <phlib/xl_ufile.h>
#define  UNIQUE_BUF_LEN 50

/* ------------------------------------------------------------------------- */
/* Return a unique filename into a preallocated buffer 
 */
RetValue 
xl_unique_filename(char *name,const char *unique_filename_file,size_t flen)
{
  /* Nasty! have to use file descriptors to use locking... */
  int     ufile = 0;
  int     cpos  = 0;
  flock_t lock;
  char    name_buffer[UNIQUE_BUF_LEN];

  /* Open the lock file... */
  if( (ufile = open(unique_filename_file,O_RDWR)) == -1 ) {
    /* Touch the file as it doesn't exist,
       (locking doesn't release on O_CREAT) */
    creat(unique_filename_file,S_IFREG|0666);
    ufile=open(unique_filename_file,O_RDWR);
  }

  if( ufile == -1 ) {
    /* Aha... an error. We must return it */
    return Failure;
  }

  /* Lock for critical section! */ 
  lock.l_type   = F_WRLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start  = 0;
  lock.l_len    = flen;

  if( fcntl( ufile, F_SETLKW, &lock ) == -1 ) {
    close(ufile);
    return Failure;
  }
    
  /* Get the current count... */
  lseek( ufile, 0L, SEEK_SET );
  if( read( ufile, (void*)name_buffer, flen ) != flen ) {
    /* File name not made... reset count to 0 */
    memset(name_buffer,'0',flen);
  }

  name_buffer[flen]='\0';
    
    /* Set the return string */
  memcpy(name,name_buffer,flen+1);    

  /* Update the count... */
  cpos=flen;
  do {
    cpos--;
    name_buffer[cpos]++;
    if( name_buffer[cpos] > '9' ) {
      name_buffer[cpos]='0';
    }
  } while ((name_buffer[cpos]=='0')&&(cpos>0));

  /* Write it back... */
  lseek(ufile,0L,SEEK_SET);
  write(ufile,(void*)name_buffer,flen);

    /* End critical section - lock removed when file is closed */
  close(ufile);    
  return Success;
}

/* ------------------------------------------------------------------------- */
/* end */
